--[[
 MiniPet Addon
 Version 3.0
 $Revision: 74747 $

 Currently maintained by Breanni of Scarlet Crusade
 All credit for authoring this addon is listed below

 Visit WarcraftPets.com for all your minipet needs!
]]--

--[[
    MiniPet by Carra of Stormrage
    
    There are a number of events where the player looses his pet and we can notify him:
    -Entering the game                   >OK
    -Entering an instance                >OK
    -Dieing and getting back alive       >Usually OK :p
    -Using a Taxi Service                >OK
    -Teleporting/Summoning/Hearthstoning >OK but no summoning
    
    Based originally on MiniPetLeash (completely rewritten except for the events which are copied).
    
    Updated and converted to Ace2 by LordFarlander
]]--

local L = AceLibrary( "AceLocale-2.2" ):new( "MiniPet" );

local options = {
        type = "group",
        args = {
            [L["toggleAutoHideButton"]] = {
                type = "toggle",
                name = L["Auto Hide Button"],
                desc = L["Hide the button when you clicked it?"],
                get = function()
                          return MiniPet.db.char.AutoHideButton;
                      end,
                set = function( val )
                          MiniPet.db.char.AutoHideButton = not MiniPet.db.char.AutoHideButton;
                      end,
            },
--         toggleDebug = {
--             type = "toggle",
--             name = "Debug",
--             desc = "Debug mode toggle",
--             get = function()
--                       return MiniPet.DEBUG;
--                   end,
--             set = function( val )
--                       MiniPet.DEBUG = not self.DEBUG;
--                   end,
--         },
            [L["toggleButton"]] = {
                type = "toggle",
                name = L["Show Button"],
                desc = L["Show/Hide the MiniPet-Button."],
                get = function()
                          return MiniPet_Button:IsVisible()
                      end,
                set = function( val )
                          if( MiniPet_Button:IsVisible() ) then
                              MiniPet_Button:Hide();
                          else
                              MiniPet_Button:Show();
                          end
                      end,
            },
            [L["toggleTextWarning"]] = {
                type = "toggle",
                name = L["Text Warning"],
                desc = L["Show text warnings to warn you get out a minipet?"],
                get = function()
                          return MiniPet.db.char.GiveTextWarning;
                      end,
                set = function( val )
                          MiniPet.db.char.GiveTextWarning = not MiniPet.db.char.GiveTextWarning;
                      end,
            },
            [L["toggleButtonWarning"]] = {
                type = "toggle",
                name = L["Button Warning"],
                desc = L["Flash the button to warn you to get out a minipet?"],
                get = function()
                          return MiniPet.db.char.GiveButtonWarning;
                      end,
                set = function( val )
                          MiniPet.db.char.GiveButtonWarning = not MiniPet.db.char.GiveButtonWarning;
                      end,
            },
            [L["toggleMountWarning"]] = {
                type = "toggle",
                name = L["Mount Warning"],
                desc = L["Show text warnings to warn you get out a minipet when getting of a flying mount?"],
                get = function()
                          return MiniPet.db.char.GiveMountWarning;
                      end,
                set = function( val )
                          MiniPet.db.char.GiveMountWarning = not MiniPet.db.char.GiveMountWarning;
                      end,
            },
            [L["toggleBG"]] = {
                type = "toggle",
                name = L["Battle Ground Check"],
                desc = L["Show warnings in a battleground?"],
                get = function()
                          return MiniPet.db.char.BattleGroundCheck;
                      end,
                set = function( val )
                          MiniPet.db.char.BattleGroundCheck = not MiniPet.db.char.BattleGroundCheck;
                      end,
            },
            [L["lockButton"]] = {
                type = "toggle",
                name = L["Lock Button Position"],
                desc = L["Lock the position of the button?"],
                get = function()
                          return MiniPet.db.char.LockButton;
                      end,
                set = function( val )
                          MiniPet.db.char.LockButton = not MiniPet.db.char.LockButton;
                      end,
            },
            [L["list"]] = {
                type = "execute",
                name = L["list"],
                desc = L["Show a list with all your pets."],
                func = "ListPets",
            },
            [L["relist"]] = {
                type = "execute",
                name = L["relist"],
                desc = L["Force a refresh of the list of all your pets."],
                func = "GetPlayersPets",
            },
        },
    };

MiniPet = AceLibrary( "AceAddon-2.0" ):new( "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0" );
MiniPet.version = tonumber( string.match( "$Rev: 74747 $", "(%d+)" ) );
MiniPet:RegisterChatCommand( "/MiniPet", "/mp", options );
MiniPet:RegisterDB( "MiniPetDB", "MiniPetDBPC" );
MiniPet:RegisterDefaults( "char", {
        AutoHideButton = true,
        BattleGroundCheck = true,
        GiveTextWarning = true,
        GiveButtonWarning = true,
        GiveMountWarning = true,
        LockButton = false,
    } );
    
local PetAndMountDatabase = LibStub( "PetAndMountDatabase-1.0" );
        
--Global Values
MiniPet.DEBUG               = false; --true = Debugging Mode
MiniPet.UpdateInterval      = 0.2;   --Time between two flashes of the button in seconds
MiniPet.TotalFlashTime      = 10;    --Total time to flash the button in seconds

--DO NOT CHANGE THESE VALUES
MiniPet.PlayersPets         = {};    --List of the players pet
MiniPet.CurrentPet          = nil;   --Pet on the Button
MiniPet.BoardedTaxi         = false;
MiniPet.StartedHearthstone  = false;
MiniPet.HearthstoneComplete = false;
MiniPet.PlayerDied          = false;
MiniPet.PlayerEnteringWorld = true;
MiniPet.ReadyToInitialize   = false;
MiniPet.PetListOkay         = false;

MiniPet.ShowButtonImg       = false; --Goes on and off if button flashes
MiniPet.FlashTime           = 0;     --Time since button flashes
MiniPet.FlashButton         = false; --On when the button flashes

local MINIPET_TELEPORTS = {
        L["Hearthstone"],
        L["Ruby Slippers"],
        --A Mage Teleports
        L["Teleport: Darnassus"],
        L["Teleport: Ironforge"],
        L["Teleport: Stormwind"],
        L["Teleport: Exodar"],
        L["Teleport: Theramore"],
        --H Teleports
        L["Teleport: Undercity"],
        L["Teleport: Orgrimmar"],
        L["Teleport: Thunder Bluff"],
        L["Teleport: Silvermoon"],
        L["Teleport: Stonard"],
        --Shattrath
        L["Teleport: Shattrath"],
        --Druid Teleport
        L["Teleport: Moonglade"],
        --Engineering spells
        L["Gnomish Transporter"],
        L["Everlook Transporter"],
        L["Toshley's Station Transporter"],
        L["Area52 Transporter"],
    };
    
local MINIPET_BATTLEGROUNDS = {
    	L["Warsong Gulch"],
    	L["Arathi Basin"],
    	L["Alterac Valley"],
    	L["Eye of the Storm"],
	};

function MiniPet:OnEnable()
    --Register the Events
    self:RegisterEvent( "BAG_UPDATE",
        function( bagID )
            if( MiniPet:HasMiniPetListChanged() )
                then MiniPet:ChooseRandomPet();
            end--if
        end );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD",
        function()
            MiniPet.PlayerDied = UnitIsDeadOrGhost( "player" );
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            MiniPet.PlayerEnteringWorld = false;
            MiniPet:WarnPlayer();
        end );
    self:RegisterEvent( "PLAYER_CONTROL_LOST",
        function()
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
        end );
    self:RegisterEvent( "PLAYER_CONTROL_GAINED",
        function()
            if( MiniPet.BoardedTaxi ) then
                MiniPet.BoardedTaxi = false;
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_DEAD",
        function()
            MiniPet:dprint( "MiniPet: PlayerDied" );
            MiniPet.PlayerDied = true;
        end );
    self:RegisterEvent( "PLAYER_ALIVE",
        function()
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            if( MiniPet.HearthstoneComplete ) then
                MiniPet.HearthstoneComplete = false;
                MiniPet:dprint( "MiniPet: Activating Pet After Hearthstone!" );
                MiniPet:WarnPlayer();
            end--if
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer dead!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_UNGHOST",
        function()
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer ghost!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED",
        function( unit, _, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    MiniPet:dprint( "MiniPet: Interrupted Teleporting!" );
                    MiniPet.StartedHearthstone = false;
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED",
        function( unit, spellName, _ )
            if( unit == "player" ) then
                if( self.StartedHearthstone ) then
                     for _, spell in pairs( MINIPET_TELEPORTS ) do
                        if( spellName == spell ) then
                            MiniPet:dprint( "MiniPet: Finished Teleporting!" );
                            MiniPet.StartedHearthstone = false;
                            MiniPet.HearthstoneComplete = true;
                            MiniPet:WarnPlayer();
                        end--if
                    end--if
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SENT",
        function( unit, spellName, _, _ )
            if( unit == "player" ) then
                MiniPet:dprint( "Spellcast: " .. spellName );
                MiniPet.StartedHearthstone = false;
                for _, spell in pairs( MINIPET_TELEPORTS ) do
                    if( spellName == spell ) then
                        MiniPet:dprint( "MiniPet: Started Teleporting!" );
                        MiniPet.StartedHearthstone = true;
                    end--if
                end--for
            end--if
        end );
    self:RegisterEvent( "SpecialEvents_Dismounted",
        function( mountName, speed, flying )
            if( flying and MiniPet.db.char.GiveMountWarning ) then
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            MiniPet:SetButton( self.currentPet );
        end );

    --Upgrade check
    if( self.db.char.LastVersion == nil ) then
        --Upgrading from 2.0 or earlier, import the old settings
        self.db.char.BattleGroundCheck = MiniPet_BattleGroundCheck;
        self.db.char.GiveTextWarning = MiniPet_GiveTextWarning;
        self.db.char.GiveButtonWarning = MiniPet_GiveButtonWarning;
        self.db.char.AutoHideButton = MiniPet_HideButton;
    end--if
    if( self.db.char.LastVersion ~= self.version ) then
        self.db.char.LastVersion = self.version;
    end--if

    self.cooldown = CreateFrame( "Cooldown", MiniPet_Button:GetName() .. "Cooldown", MiniPet_Button, "CooldownFrameTemplate" );
    self.cooldown:SetReverse( false );
    self.cooldown:SetAllPoints( MiniPet_Button );
    self.cooldown:Hide();

    --Create the Pet Inventory
    self:GetPlayersPets();
    --Set a Pet
    self:ChooseRandomPet();
end--MiniPet:OnEnable();


function MiniPet:OnUpdate( elapsed )
    if( ( UnitName( "player" ) ~= UNKNOWNOBJECT ) and self.ReadyToInitialize ) then
        self:dprint( "MiniPet: PlayerEnteringWorld and ALIVE" );
        self.PlayerEnteringWorld = false;
        self.ReadyToInitialize = false;
        self:WarnPlayer();
    end

    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    if( self.FlashButton ) then
        if( MiniPet_Button.TimeSinceLastUpdate == nil ) then
            MiniPet_Button.TimeSinceLastUpdate = 0;
            self.FlashTime = 0;
        end--if
        MiniPet_Button.TimeSinceLastUpdate = MiniPet_Button.TimeSinceLastUpdate + elapsed;
        self.FlashTime = self.FlashTime + elapsed;
        if( self.FlashTime < self.TotalFlashTime) then
            if( MiniPet_Button.TimeSinceLastUpdate > self.UpdateInterval ) then
                if( self.ShowButtonImg and ( self.CurrentPet ~= nil ) ) then
                    local _, _, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( self.CurrentPet );

                    self:SetButtonImage( sItemTexture );
                    self.ShowButtonImg = false;
                else
                    self.SetButtonImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );
                    self.ShowButtonImg = true;
                end--if
                MiniPet_Button.TimeSinceLastUpdate = 0;
            end--if
        else
            self.FlashButton = false;
            self:SetButton( self.CurrentPet );
            self.FlashTime = 0;
        end--if
    end--if
end--MiniPet:OnUpdate( elapsed )

----------------------------
--+Pet/Inventory Functions--
--Pick a random pet and set the button.
function MiniPet:ChooseRandomPet()
    local amountOfPets = #self.PlayersPets;
    local playerPetList = self.PlayersPets;    

    if( amountOfPets > 0 ) then
        --math.random(upper) generates integer numbers between 1 and upper.
        local randomPetIndex = math.random( amountOfPets );

        if( amountOfPets > 1 ) then --Lets make sure the player gets a different pet if he gots more then one, the quick and dirty way.
            while( playerPetList[randomPetIndex] == self.CurrentPet ) do
                randomPetIndex = math.random( amountOfPets );
            end--while
        end--if
        self.CurrentPet = playerPetList[randomPetIndex];
    else
        self.CurrentPet = nil;
    end
    self:SetButton( self.CurrentPet );
end--MiniPet:ChooseRandomPet()

--Gets all your pets from your inventory.
function MiniPet:GetPlayersPets()
    local miniPets = PetAndMountDatabase.Collections.MiniPets;
    
    self.PlayersPets = {};

    local playerPetList = self.PlayersPets;

    for _, sublist in pairs( miniPets ) do
        local iterator, handle = PetAndMountDatabase:IterateSet( sublist );
        local pet = iterator( handle );

        while pet do
            if( GetItemCount( pet ) > 0 ) then
                table.insert( playerPetList, pet );
            end--if
            pet = iterator( handle );
        end--for
    end--for
end--MiniPet:GetPlayerPets()

--Prints out a list with all your pets.
function MiniPet:ListPets()
    local playerPetList = self.PlayersPets;

    self:Print( L["MiniPet, List of your pets:"] );
    for _, pet in pairs( playerPetList ) do
        local sPetName, _, _, _, _, _, _, _ = GetItemInfo( pet );

        self:Print( "-|cFF00FFFF" .. sPetName );
    end--for
end--MiniPet:listPets()
---Pet/Invenroty Functions--
----------------------------

-------------------
--+Help Functions--
function MiniPet:CopyTable( to, from )
    for _, v in pairs( from ) do
        table.insert( to, v );
    end -- for
end--MiniPet:CopyTable( to, from )

function MiniPet:CompareTables( table1, table2 )
    for i, v in pairs( table1 ) do
        if( table2[i] ~= v ) then
            self:dprint( "table 2 did not have value " .. v );
            return false;
        end--if
    end -- for
    for i, v in pairs( table2 ) do
        if( table1[i] ~= v ) then
            self:dprint( "table 1 did not have value " .. v );
            return false;
        end--if
    end -- for
    return true;
end--MiniPet:CompareTables( table1, table2 )

function MiniPet:HasMiniPetListChanged()
    local prePetCount = #self.PlayersPets;
    local prePetList =  {};

    self:CopyTable( prePetList, self.PlayersPets );

    self:GetPlayersPets();

    local afterPetCount = #self.PlayersPets;

    -- differences in sizes always means the list has changed
    if( ( prePetCount ~= afterPetCount ) or ( not self.PetListOkay ) ) then
        self.PetListOkay = true;
        self:dprint( "Number of minipets changed" );
        return true;
    end--if
    self.PetListOkay = true;
    return not self:CompareTables( prePetList, self.PlayersPets );
end--MiniPet:HasMiniPetListChanged()

--Check if a player is in a battleground
function MiniPet:isInBg()
    for _, bg in pairs( MINIPET_BATTLEGROUNDS ) do
        if( GetZoneText() == bg ) then
            return true;
        end--if
    end--for
    return false;
end--MiniPet:isInBg()
---Help Functions--
-------------------

----------------------
--+Message Functions--
--Give the player a reminder.
function MiniPet:WarnPlayer()
    if( ( not ( self:isInBg() and self.db.char.BattleGroundCheck ) ) and ( not self.PlayerDied ) and ( not self.BoardedTaxi ) ) then--BG + option check
        if( self.db.char.GiveButtonWarning == true ) then
            self.FlashButton = true;
        end--if
        if( self.db.char.GiveTextWarning ) then
            if( #self.PlayersPets > 0 ) then    --No use if player isn't cool and gots no pets!
                MiniPet:wprint( L["ALERT: Time to summon your minipet!"] );
            end--if
        end--if
        if( self.db.char.AutoHideButton and ( not MiniPet_Button:IsVisible() ) ) then
            MiniPet_Button:Show();
        end--if        
    end--if
end--MiniPet:WarnPlayer()

--Shows a warning in the middle of the screen.
function MiniPet:wprint( msg )
    MinipetTextFrameText:SetTextColor( 1.0, 0.1, 0.1 );
    MinipetTextFrameText:SetText( msg );
    MinipetTextFrame.startTime = GetTime();
    MinipetTextFrame:Show();
end--MiniPet:wprint( msg )

--Debug Print in red
function MiniPet:dprint( msg )
    if( self.DEBUG ) then
        DEFAULT_CHAT_FRAME:AddMessage( msg, 1.0, 0.0, 0.0 );
    end--if
end--dprint( msg )
---Message Functions
----------------------

-----------------
--+UI Functions--
function MiniPet:SetGameTooltips()
    local pet = MiniPet_Button:GetAttribute( "item1" );

    MiniPet_Tooltip:SetOwner( MiniPet_Button, "RIGHT" );
    MiniPet_Tooltip:SetBackdropColor( 0.0, 0.0, 0.0, 1.0 );
    if( pet == nil ) then
        MiniPet_Tooltip:SetText( L["Get a minipet for some company!"] );
    else
        MiniPet_Tooltip:SetHyperlink( pet );
    end--if
end--MiniPet:SetGameTooltips()

function MiniPet:SetButton( pet )
    local cooldownInicator = self.cooldown;

    if( not UnitAffectingCombat( "player" ) ) then
        cooldownInicator:Hide();
        if( pet == nil ) then
            MiniPet_Button:SetAttribute( "type1", "item" );
            MiniPet_Button:SetAttribute( "item1", nil );

            self:SetButtonImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );

            self:dprint( "No pets" );
        else
            local sName, itemLink, _, _, _, _, _, _, _, sItemTexture = GetItemInfo( pet );
            local _, _, itemString = itemLink:find( "^|c%x+|H(.+)|h%[.+%]" );        
            local startTime, duration, enable = GetItemCooldown( itemLink );
    
            MiniPet_Button:SetAttribute( "type1", "item" );
            MiniPet_Button:SetAttribute( "item1", itemString );
    
            self:SetButtonImage( sItemTexture );
    
            if( sName == nil ) then
                self:Print( "Something is wrong with pet " .. pet );
            else
                self:dprint( "Pet set: " .. sName );
            end--if
            
            if( enable ) then
                cooldownInicator:Show();
                cooldownInicator:SetCooldown( startTime, duration );
            else
                cooldownInicator:Hide();
            end--if
        end--if
    end--if
end--MiniPet:SetButton( pet )

function MiniPet:SetButtonImage( img )
    MiniPet_Button:SetNormalTexture( img );
    MiniPet_Button:SetHighlightTexture( img );
end--MiniPet:SetButtonImage( img )
---UI Functions--
-----------------
